/**********************************************************
　　製作者　　アドウィン
　　解　説　　ドットマトリクス LED の制御用の関数類
**********************************************************/
#include <Arduino.h> // Arduino.h の読み込み

// IOエクスパンダ使用（使わない場合はこの行をコメントアウト）
#define IOEXPANDER

#ifdef IOEXPANDER

#include <Wire.h> // Wire.h の読み込み

// IOエクスパンダのI2C上でのアドレス
int ioxAdr = 0x27;

/*
 * ドットマトリクスLED初期設定関数
 */
void initLed() {

  // I2Cの初期化
  Wire.begin();

  // IOエクスパンダの初期設定
  Wire.beginTransmission(ioxAdr); // IOエクスパンダとの通信の開始
  Wire.write(0x00);               // レジスタのアドレスの設定（0x00=GPIOA入出力設定レジスタ）
  Wire.write(0x00);               // GPIOAのピンを出力に設定（0x00=GPIOA入出力設定レジスタに0x00を書き込み）
  Wire.write(0x00);               // GPIOBのピンを出力に設定（0x01=GPIOB入出力設定レジスタに0x00を書き込み）
  Wire.endTransmission();         // データを送信し通信を完了

  // IOエクスパンダのピンの初期値の設定
  Wire.beginTransmission(ioxAdr); // IOエクスパンダとの通信の開始
  Wire.write(0x12);               // レジスタのアドレスの設定（0x12=GPIOA出力レベル設定レジスタ）
  Wire.write(0b00000000);         // GPIOAのピンの出力レベルの設定（0x12=GPIOA出力レベル設定レジスタ）
  Wire.write(0b11111111);         // GPIOBのピンの出力レベルの設定（0x13=GPIOB出力レベル設定レジスタ）
  Wire.endTransmission();         // データを送信し通信を完了
}

/*
 * ドットマトリクスLED点灯パターン設定関数
 * 引数: patternAnode 表示するパターン（アノード側）
 *       patternCathode 表示するパターン（カソード側）
 */
void setLed(byte patternAnode, byte patternCathode) {
  Wire.beginTransmission(ioxAdr); // IOエクスパンダとの通信の開始
  Wire.write(0x13);               // レジスタのアドレスの設定（0x13=GPIOB出力レベル設定レジスタ）
  Wire.write(0xFF);               // カソード側の出力レベルを全消灯に設定
  Wire.endTransmission();         // 送信の完了
  Wire.beginTransmission(ioxAdr); // IOエクスパンダとの通信の開始
  Wire.write(0x12);               // レジスタのアドレスの設定（0x12=GPIOA出力レベル設定レジスタ）
  Wire.write(patternAnode);       // アノード側の出力レベルの設定（0x12=GPIOA出力レベル設定レジスタ）
  Wire.write(~patternCathode);    // カソード側の出力レベルの設定（0x13=GPIOB出力レベル設定レジスタ）
  Wire.endTransmission();         // 送信の完了
}

#else

// ドットマトリクスLEDアノード側ピンの配列
int ledAnodePins[8] = {0, 2, 4, 5, 16, 17, 18, 19};
// ドットマトリクスLEDカソード側ピンの配列
int ledCathodePins[8] = {21, 22, 23, 25, 26, 27, 32, 33};

/*
 * ドットマトリクスLED初期設定関数
 */
void initLed() {

  // 各アノード・カソード端子の設定
  for (int i = 0; i < 8; i++) {

    // 入出力設定
    pinMode(ledAnodePins[i], OUTPUT);
    pinMode(ledCathodePins[i], OUTPUT);

    // 出力レベル設定（全消灯）
    digitalWrite(ledAnodePins[i], LOW);
    digitalWrite(ledCathodePins[i], HIGH);
  }
}

/*
 * ドットマトリクスLED点灯パターン設定関数
 * 引数: patternAnode 表示するパターン（ アノード側）
 *       patternCathode 表示するパターン（ カソード側）
 */
void setLed(byte patternAnode, byte patternCathode) {

  // カソード側のパターンを反転
  patternCathode = ~patternCathode;

  // 列および行の点灯設定
  for (int i = 0; i < 8; i++) {
    // i 列目に patternAnode の i ビット目の値を設定
    digitalWrite(ledAnodePins[i], bitRead(patternAnode, i));
    // i 行目に patternCathode の i ビット目の値を設定
    digitalWrite(ledCathodePins[i], bitRead(patternCathode, i));
  }
}
#endif