/**********************************************************
　　製作者　　アドウィン
　　解　説　　ロータリエンコーダ用の関数類
**********************************************************/
#include <Arduino.h>     // Arduino.hの読み込み
#include <driver/pcnt.h> // pcnt.h の読み込み

/*
 * ロータリエンコーダの初期設定関数
 * 引数: encAPin A相入力のピン番号
 *       encBPin B相入力のピン番号
 *       multi 逓倍数（1、2、4のいずれか。初期値1。無効な値の場合も1扱い。）
 */
void initEnc(int encAPin, int encBPin, int multi) {

  // ロータリエンコーダ用ピンの初期設定
  pinMode(encAPin, INPUT_PULLUP);
  pinMode(encBPin, INPUT_PULLUP);

  // PCNTの設定用の構造体の宣言（1倍逓信用）
  pcnt_config_t pcnt_config = {
      .pulse_gpio_num = encAPin,       // パルス入力にA相を設定
      .ctrl_gpio_num = encBPin,        // コントロール信号にB相を設定
      .lctrl_mode = PCNT_MODE_KEEP,    // コントロール信号がLレベルの時、通常のカウント
      .hctrl_mode = PCNT_MODE_DISABLE, // コントロール信号がHレベルの時、カウントしない
      .pos_mode = PCNT_COUNT_INC,      // パルス入力の立ち上がりでカウントを増やす
      .neg_mode = PCNT_COUNT_DEC,      // パルス入力の立ち下がりでカウントを減らす
      .counter_h_lim = 32767,          // カウントの最大値を32767に設定
      .counter_l_lim = -32768,         // カウントの最小値を-32768に設定
      .unit = PCNT_UNIT_0,             // PCNTユニットとしてユニット0を使用
      .channel = PCNT_CHANNEL_0,       // PCNTユニットのチャンネル0を使用
  };
  // 2逓倍または4逓倍の時
  if (2 == multi || 4 == multi) {
    // 2逓倍用に設定変更
    pcnt_config.hctrl_mode = PCNT_MODE_REVERSE; // コントロール信号がHレベルの時、通常と逆のカウント
  }
  // PCNTの設定（ユニット0チャンネル0）
  pcnt_unit_config(&pcnt_config);
  // 4逓倍の時
  if (4 == multi) {
    // 4逓倍用にチャンネル1を設定
    pcnt_config.pulse_gpio_num = encBPin;  // パルス入力をB相に変更
    pcnt_config.ctrl_gpio_num = encAPin;   // コントロール信号をA相に変更
    pcnt_config.pos_mode = PCNT_COUNT_DEC; // パルス入力の立ち上がりでカウントを減らす
    pcnt_config.neg_mode = PCNT_COUNT_INC; // パルス入力の立ち下がりでカウントを増やす
    pcnt_config.channel = PCNT_CHANNEL_1;  // PCNTユニットのチャンネル1を使用
    // PCNTの設定（ユニット0チャンネル1）
    pcnt_unit_config(&pcnt_config);
  }
  // PCNTのユニット0のカウンタ初期化
  pcnt_counter_pause(PCNT_UNIT_0);  // ユニット0のカウンタを一時停止
  pcnt_counter_clear(PCNT_UNIT_0);  // ユニット0のカウンタを初期化
  pcnt_counter_resume(PCNT_UNIT_0); // ユニット0のカウンタを再開
}

/*
 * ロータリエンコーダのカウント数の取得関数
 */
int16_t getEncCount(void) {

  // カウント数の取得
  int16_t count;
  pcnt_get_counter_value(PCNT_UNIT_0, &count);
  // カウント数を返す
  return count;
}